@@ -31,6 +31,7 @@
 #include "value.h"
 #include "gjs/byteArray.h"
 #include <gjs/gjs.h>
+#include <gjs/compat.h>
 
 #include <util/log.h>
 
@@ -298,7 +299,7 @@ gjs_object_to_g_hash(JSContext   *context,
     if (iter == NULL)
         return JS_FALSE;
 
-    prop_id = JSVAL_VOID;
+    prop_id = JSID_VOID;
     if (!JS_NextProperty(context, iter, &prop_id))
         return JS_FALSE;
 
@@ -307,7 +308,7 @@ gjs_object_to_g_hash(JSContext   *context,
      * Rely on the type-aware g_argument_release functions. */
    result = g_hash_table_new(g_str_hash, g_str_equal);
 
-    while (prop_id != JSVAL_VOID) {
+   while (!JSID_IS_VOID(prop_id)) {
         jsval key_js, val_js;
         GArgument key_arg, val_arg;
 
@@ -340,7 +341,7 @@ gjs_object_to_g_hash(JSContext   *context,
 
         g_hash_table_insert(result, key_arg.v_pointer, val_arg.v_pointer);
 
-        prop_id = JSVAL_VOID;
+        prop_id = JSID_VOID;
         if (!JS_NextProperty(context, iter, &prop_id))
             goto free_hash_and_fail;
     }
@@ -363,14 +364,14 @@ gjs_array_from_strv(JSContext   *context,
     guint i;
     JSBool result = JS_FALSE;
 
-    obj = JS_NewArrayObject(context, 0, JSVAL_NULL);
+    obj = JS_NewArrayObject(context, 0, NULL);
     if (obj == NULL)
         return JS_FALSE;
 
     *value_p = OBJECT_TO_JSVAL(obj);
 
     elem = JSVAL_VOID;
-    JS_AddRoot(context, &elem);
+    JS_AddValueRoot(context, &elem);
 
     for (i = 0; strv[i] != NULL; i++) {
         if (!gjs_string_from_utf8 (context, strv[i], -1, &elem))
@@ -385,7 +386,7 @@ gjs_array_from_strv(JSContext   *context,
     result = JS_TRUE;
 
 out:
-    JS_RemoveRoot(context, &elem);
+    JS_RemoveValueRoot(context, &elem);
 
     return result;
 }
@@ -1373,14 +1374,14 @@ gjs_array_from_g_list (JSContext  *context,
     GArgument arg;
     JSBool result;
 
-    obj = JS_NewArrayObject(context, 0, JSVAL_NULL);
+    obj = JS_NewArrayObject(context, 0, NULL);
     if (obj == NULL)
         return JS_FALSE;
 
     *value_p = OBJECT_TO_JSVAL(obj);
 
     elem = JSVAL_VOID;
-    JS_AddRoot(context, &elem);
+    JS_AddValueRoot(context, &elem);
 
     result = JS_FALSE;
 
@@ -1420,7 +1421,7 @@ gjs_array_from_g_list (JSContext  *context,
     result = JS_TRUE;
 
  out:
-    JS_RemoveRoot(context, &elem);
+    JS_RemoveValueRoot(context, &elem);
 
     return result;
 }
@@ -1441,7 +1442,7 @@ gjs_array_from_g_array (JSContext  *context,
     element_type = g_type_info_get_tag(param_info);
     element_type = replace_gtype(element_type);
 
-    obj = JS_NewArrayObject(context, 0, JSVAL_NULL);
+    obj = JS_NewArrayObject(context, 0, NULL);
     if (obj == NULL)
       return JS_FALSE;
 
@@ -1509,7 +1510,7 @@ gjs_array_from_g_array (JSContext  *context,
     result = JS_TRUE;
 
 finally:
-    JS_RemoveRoot(context, &elem);
+    JS_RemoveValueRoot(context, &elem);
 
     return result;
 }
@@ -1540,16 +1541,16 @@ gjs_object_from_g_hash (JSContext  *context,
         return JS_FALSE;
 
     *value_p = OBJECT_TO_JSVAL(obj);
-    JS_AddRoot(context, &obj);
+    JS_AddObjectRoot(context, &obj);
 
     keyjs = JSVAL_VOID;
-    JS_AddRoot(context, &keyjs);
+    JS_AddValueRoot(context, &keyjs);
 
     valjs = JSVAL_VOID;
-    JS_AddRoot(context, &valjs);
+    JS_AddValueRoot(context, &valjs);
 
     keystr = NULL;
-    JS_AddRoot(context, &keystr);
+    JS_AddStringRoot(context, &keystr);
 
     result = JS_FALSE;
 
@@ -1583,10 +1584,10 @@ gjs_object_from_g_hash (JSContext  *context,
 
  out:
     if (keyutf8) g_free(keyutf8);
-    JS_RemoveRoot(context, &obj);
-    JS_RemoveRoot(context, &keyjs);
-    JS_RemoveRoot(context, &valjs);
-    JS_RemoveRoot(context, &keystr);
+    JS_RemoveObjectRoot(context, &obj);
+    JS_RemoveValueRoot(context, &keyjs);
+    JS_RemoveValueRoot(context, &valjs);
+    JS_RemoveStringRoot(context, &keystr);
 
     return result;
 }
@@ -1642,10 +1643,10 @@ gjs_value_from_g_argument (JSContext  *context,
         return JS_NewNumberValue(context, arg->v_int8, value_p);
 
     case GI_TYPE_TAG_FLOAT:
-        return JS_NewDoubleValue(context, arg->v_float, value_p);
+        return JS_NewNumberValue(context, arg->v_float, value_p);
 
     case GI_TYPE_TAG_DOUBLE:
-        return JS_NewDoubleValue(context, arg->v_double, value_p);
+        return JS_NewNumberValue(context, arg->v_double, value_p);
 
     case GI_TYPE_TAG_FILENAME:
         if (arg->v_pointer)
