@@ -248,7 +248,6 @@ gjs_closure_invoke(GClosure *closure,
     c = (Closure*) closure;
 
     check_context_valid(c);
-    context = c->context;
 
     if (c->obj == NULL) {
         /* We were destroyed; become a no-op */
@@ -256,12 +255,13 @@ gjs_closure_invoke(GClosure *closure,
         return;
     }
 
+    context = gjs_runtime_get_current_context(c->runtime);
     JS_BeginRequest(context);
 
     if (JS_IsExceptionPending(context)) {
         gjs_debug_closure("Exception was pending before invoking callback??? "
                           "Not expected");
-        gjs_log_exception(c->context, NULL);
+        gjs_log_exception(context, NULL);
     }
 
     if (!gjs_call_function_value(context,
@@ -276,26 +276,35 @@ gjs_closure_invoke(GClosure *closure,
                           closure, c->obj);
         if (!gjs_log_exception(context, NULL))
             gjs_debug_closure("Closure invocation failed but no exception was set?");
-        return;
+        goto out;
     }
 
     if (gjs_log_exception(context, NULL)) {
         gjs_debug_closure("Closure invocation succeeded but an exception was set");
     }
 
+ out:
     JS_EndRequest(context);
 }
 
-JSContext*
-gjs_closure_get_context(GClosure *closure)
+gboolean
+gjs_closure_is_valid(GClosure *closure)
 {
     Closure *c;
 
     c = (Closure*) closure;
 
-    check_context_valid(c);
+    return c->context != NULL;
+}
 
-    return c->context;
+JSRuntime*
+gjs_closure_get_runtime(GClosure *closure)
+{
+    Closure *c;
+
+    c = (Closure*) closure;
+
+    return c->runtime;
 }
 
 JSObject*
@@ -317,12 +326,13 @@ gjs_closure_new(JSContext  *context,
 
     c = (Closure*) g_closure_new_simple(sizeof(Closure), NULL);
     c->runtime = JS_GetRuntime(context);
-    /* Closure are executed in our special "load-context" (one per runtime).
-     * This ensures that the context is still alive when the closure
-     * is invoked (as long as the runtime lives)
+    /* The saved context is used for lifetime management, so that the closure will
+     * be torn down with the context that created it. The context could be attached to
+     * the default context of the runtime using if we wanted the closure to survive
+     * the context that created it.
      */
-    c->context = gjs_runtime_get_load_context(c->runtime);
-    JS_BeginRequest(c->context);
+    c->context = context;
+    JS_BeginRequest(context);
 
     c->obj = callable;
     c->unref_on_global_object_finalized = FALSE;
@@ -333,7 +343,7 @@ gjs_closure_new(JSContext  *context,
      */
     g_closure_add_finalize_notifier(&c->base, NULL, closure_finalized);
 
-    gjs_keep_alive_add_global_child(c->context,
+    gjs_keep_alive_add_global_child(context,
                                     global_context_finalized,
                                     c->obj,
                                     c);
@@ -343,7 +353,7 @@ gjs_closure_new(JSContext  *context,
     gjs_debug_closure("Create closure %p which calls object %p '%s'",
                       c, c->obj, description);
 
-    JS_EndRequest(c->context);
+    JS_EndRequest(context);
 
     return &c->base;
 }
