@@ -28,10 +28,13 @@
 
 #include <util/log.h>
 #include <gjs/gjs.h>
+#include <gjs/compat.h>
 
 static char **include_path = NULL;
+static char *command = NULL;
 
 static GOptionEntry entries[] = {
+    { "command", 'c', 0, G_OPTION_ARG_STRING, &command, "Program passed in as a string", "COMMAND" },
     { "include-path", 'I', 0, G_OPTION_ARG_STRING_ARRAY, &include_path, "Add the directory DIR to the list of directories to search for js files.", "DIR" },
     { NULL }
 };
@@ -76,9 +79,13 @@ main(int argc, char **argv)
                                          &error)) {
         g_printerr("Failed to defined ARGV: %s", error->message);
         exit(1);
-    } 
+    }
 
-    if (argc <= 1) {
+    if (command != NULL) {
+        script = command;
+        len = strlen(script);
+        filename = "<command line>";
+    } else if (argc <= 1) {
         script = g_strdup("const Console = imports.console; Console.interact();");
         len = strlen(script);
         filename = "<stdin>";
@@ -99,7 +106,7 @@ main(int argc, char **argv)
         g_printerr("%s\n", error->message);
         exit(1);
     }
-    
+
     gjs_memory_report("before destroying context", FALSE);
     g_object_unref(js_context);
     gjs_memory_report("after destroying context", TRUE);
