@@ -317,6 +317,7 @@ gjs_string_get_binary_data(JSContext       *context,
     if (!JSVAL_IS_STRING(value)) {
         gjs_throw(context,
                   "Value is not a string, can't return binary data from it");
+        JS_EndRequest(context);
         return JS_FALSE;
     }
 
@@ -420,7 +421,8 @@ gjs_string_get_uint16_data(JSContext       *context,
 
 /**
  * gjs_get_string_id:
- * @id_val: a jsval that is an object hash key (could be an int or string)
+ * @context: a #JSContext
+ * @id: a jsid that is an object hash key (could be an int or string)
  * @name_p place to store ASCII string version of key
  *
  * If the id is not a string ID, return false and set *name_p to %NULL.
@@ -429,9 +431,15 @@ gjs_string_get_uint16_data(JSContext       *context,
  * Returns: true if *name_p is non-%NULL
  **/
 JSBool
-gjs_get_string_id (jsval            id_val,
+gjs_get_string_id (JSContext       *context,
+                   jsid             id,
                    const char     **name_p)
 {
+    jsval id_val;
+
+    if (!JS_IdToValue(context, id, &id_val))
+        return JS_FALSE;
+
     if (JSVAL_IS_STRING(id_val)) {
         *name_p = JS_GetStringBytes(JSVAL_TO_STRING(id_val));
         return JS_TRUE;
@@ -443,44 +451,28 @@ gjs_get_string_id (jsval            id_val,
 
 
 #if GJS_BUILD_TESTS
+#include "unit-test-utils.h"
 #include <string.h>
 
 
-static void
-test_error_reporter(JSContext     *context,
-                    const char    *message,
-                    JSErrorReport *report)
-{
-    g_printerr("error reported by test: %s\n", message);
-}
-
 void
 gjstest_test_func_gjs_jsapi_util_string_js_string_utf8(void)
 {
+    GjsUnitTestFixture fixture;
+    JSContext *context;
     const char *utf8_string = "\303\211\303\226 foobar \343\203\237";
     char *utf8_result;
-    JSRuntime *runtime;
-    JSContext *context;
-    JSObject *global;
     jsval js_string;
 
-    runtime = JS_NewRuntime(1024*1024 /* max bytes */);
-    context = JS_NewContext(runtime, 8192);
-    JS_BeginRequest(context);
-    global = JS_NewObject(context, NULL, NULL, NULL);
-    JS_SetGlobalObject(context, global);
-    JS_InitStandardClasses(context, global);
-
-    JS_SetErrorReporter(context, test_error_reporter);
+    _gjs_unit_test_fixture_begin(&fixture);
+    context = fixture.context;
 
     g_assert(gjs_string_from_utf8(context, utf8_string, -1, &js_string) == JS_TRUE);
     g_assert(js_string);
     g_assert(JSVAL_IS_STRING(js_string));
     g_assert(gjs_string_to_utf8(context, js_string, &utf8_result) == JS_TRUE);
 
-    JS_EndRequest(context);
-    JS_DestroyContext(context);
-    JS_DestroyRuntime(runtime);
+    _gjs_unit_test_fixture_finish(&fixture);
 
     g_assert(g_str_equal(utf8_string, utf8_result));
 
@@ -490,21 +482,14 @@ gjstest_test_func_gjs_jsapi_util_string_js_string_utf8(void)
 void
 gjstest_test_func_gjs_jsapi_util_string_get_ascii(void)
 {
-    JSRuntime *runtime;
+    GjsUnitTestFixture fixture;
     JSContext *context;
-    JSObject *global;
     const char *ascii_string = "Hello, world";
     JSString  *js_string;
     jsval      void_value;
 
-    runtime = JS_NewRuntime(1024*1024 /* max bytes */);
-    context = JS_NewContext(runtime, 8192);
-    JS_BeginRequest(context);
-    global = JS_NewObject(context, NULL, NULL, NULL);
-    JS_SetGlobalObject(context, global);
-    JS_InitStandardClasses(context, global);
-
-    JS_SetErrorReporter(context, test_error_reporter);
+    _gjs_unit_test_fixture_begin(&fixture);
+    context = fixture.context;
 
     js_string = JS_NewStringCopyZ(context, ascii_string);
     g_assert(g_str_equal(gjs_string_get_ascii(STRING_TO_JSVAL(js_string)), ascii_string));
@@ -512,17 +497,14 @@ gjstest_test_func_gjs_jsapi_util_string_get_ascii(void)
     g_assert(gjs_string_get_ascii_checked(context, void_value) == NULL);
     g_assert(JS_IsExceptionPending(context));
 
-    JS_EndRequest(context);
-    JS_DestroyContext(context);
-    JS_DestroyRuntime(runtime);
+    _gjs_unit_test_fixture_finish(&fixture);
 }
 
 void
 gjstest_test_func_gjs_jsapi_util_string_get_binary(void)
 {
-    JSRuntime *runtime;
+    GjsUnitTestFixture fixture;
     JSContext *context;
-    JSObject *global;
     const char binary_string[] = "foo\0bar\0baz";
     const char binary_string_odd[] = "foo\0bar\0baz123";
     jsval js_string;
@@ -533,17 +515,11 @@ gjstest_test_func_gjs_jsapi_util_string_get_binary(void)
     g_assert_cmpuint(sizeof(binary_string), ==, 12);
     g_assert_cmpuint(sizeof(binary_string_odd), ==, 15);
 
-    runtime = JS_NewRuntime(1024*1024 /* max bytes */);
-    context = JS_NewContext(runtime, 8192);
-    JS_BeginRequest(context);
-    global = JS_NewObject(context, NULL, NULL, NULL);
-    JS_SetGlobalObject(context, global);
-    JS_InitStandardClasses(context, global);
-
-    JS_SetErrorReporter(context, test_error_reporter);
+    _gjs_unit_test_fixture_begin(&fixture);
+    context = fixture.context;
 
     js_string = JSVAL_VOID;
-    JS_AddRoot(context, &js_string);
+    JS_AddValueRoot(context, &js_string);
 
     /* Even-length string (maps nicely to len/2 jschar) */
     if (!gjs_string_from_binary_data(context,
@@ -574,7 +550,7 @@ gjstest_test_func_gjs_jsapi_util_string_get_binary(void)
     g_assert(memcmp(data, binary_string_odd, len) == 0);
     g_free(data);
 
-    JS_RemoveRoot(context, &js_string);
+    JS_RemoveValueRoot(context, &js_string);
 
     void_value = JSVAL_VOID;
     g_assert(!JS_IsExceptionPending(context));
@@ -582,9 +558,7 @@ gjstest_test_func_gjs_jsapi_util_string_get_binary(void)
                                         &data, &len));
     g_assert(JS_IsExceptionPending(context));
 
-    JS_EndRequest(context);
-    JS_DestroyContext(context);
-    JS_DestroyRuntime(runtime);
+    _gjs_unit_test_fixture_finish(&fixture);
 }
 
 #endif /* GJS_BUILD_TESTS */
