@@ -24,6 +24,7 @@
 #include "dbus-values.h"
 
 #include <gjs/gjs.h>
+#include <gjs/compat.h>
 
 #include <gjs-dbus/dbus.h>
 #include <util/log.h>
@@ -51,7 +52,7 @@ gjs_js_one_value_from_dbus(JSContext       *context,
             DBusMessageIter struct_iter;
             int index;
 
-            obj = JS_NewArrayObject(context, 0, JSVAL_NULL);
+            obj = JS_NewArrayObject(context, 0, NULL);
             if (obj == NULL)
                 return JS_FALSE;
 
@@ -61,20 +62,20 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                 jsval prop_value;
 
                 prop_value = JSVAL_VOID;
-                JS_AddRoot(context, &prop_value);
+                JS_AddValueRoot(context, &prop_value);
                 if (!gjs_js_one_value_from_dbus(context, &struct_iter, &prop_value)) {
-                    JS_RemoveRoot(context, &prop_value);
+                    JS_RemoveValueRoot(context, &prop_value);
                     return JS_FALSE;
                 }
 
                 if (!JS_DefineElement(context, obj,
                                       index, prop_value,
                                       NULL, NULL, JSPROP_ENUMERATE)) {
-                    JS_RemoveRoot(context, &prop_value);
+                    JS_RemoveValueRoot(context, &prop_value);
                     return JS_FALSE;
                 }
 
-                JS_RemoveRoot(context, &prop_value);
+                JS_RemoveValueRoot(context, &prop_value);
                 dbus_message_iter_next(&struct_iter);
                 index ++;
             }
@@ -94,7 +95,7 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                 if (obj == NULL)
                     return JS_FALSE;
 
-                JS_AddRoot(context, &obj);
+                JS_AddObjectRoot(context, &obj);
                 dbus_message_iter_recurse(iter, &array_iter);
                 while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
                     DBusMessageIter entry_iter;
@@ -106,20 +107,20 @@ gjs_js_one_value_from_dbus(JSContext       *context,
 
                     if (!dbus_type_is_basic(dbus_message_iter_get_arg_type(&entry_iter))) {
                         gjs_throw(context, "Dictionary keys are not a basic type, can't convert to JavaScript");
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
                     entry_value = JSVAL_VOID;
-                    JS_AddRoot(context, &entry_value);
+                    JS_AddValueRoot(context, &entry_value);
                     if (!gjs_js_one_value_from_dbus(context, &entry_iter, &key_value)) {
-                        JS_RemoveRoot(context, &key_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &key_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
                     key_str = JS_ValueToString(context, key_value);
-                    JS_AddRoot(context, &key_str);
+                    JS_AddStringRoot(context, &key_str);
                     key = JS_GetStringBytes(key_str);
 
                     dbus_message_iter_next(&entry_iter);
@@ -127,33 +128,33 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                     gjs_debug_dbus_marshal("Defining dict entry %s in jsval dict", key);
 
                     entry_value = JSVAL_VOID;
-                    JS_AddRoot(context, &entry_value);
+                    JS_AddValueRoot(context, &entry_value);
                     if (!gjs_js_one_value_from_dbus(context, &entry_iter, &entry_value)) {
-                        JS_RemoveRoot(context, &key_value);
-                        JS_RemoveRoot(context, &key_str);
-                        JS_RemoveRoot(context, &entry_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &key_value);
+                        JS_RemoveStringRoot(context, &key_str);
+                        JS_RemoveValueRoot(context, &entry_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
                     if (!JS_DefineProperty(context, obj,
                                            key, entry_value,
                                            NULL, NULL, JSPROP_ENUMERATE)) {
-                        JS_RemoveRoot(context, &key_value);
-                        JS_RemoveRoot(context, &key_str);
-                        JS_RemoveRoot(context, &entry_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &key_value);
+                        JS_RemoveStringRoot(context, &key_str);
+                        JS_RemoveValueRoot(context, &entry_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
-                    JS_RemoveRoot(context, &key_value);
-                    JS_RemoveRoot(context, &key_str);
-                    JS_RemoveRoot(context, &entry_value);
+                    JS_RemoveValueRoot(context, &key_value);
+                    JS_RemoveStringRoot(context, &key_str);
+                    JS_RemoveValueRoot(context, &entry_value);
                     dbus_message_iter_next(&array_iter);
                 }
 
                 *value_p = OBJECT_TO_JSVAL(obj);
-                JS_RemoveRoot(context, &obj);
+                JS_RemoveObjectRoot(context, &obj);
             } else if (elem_type == DBUS_TYPE_BYTE) {
                 /* byte arrays go to a string */
                 const char *v_BYTES;
@@ -171,38 +172,38 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                 DBusMessageIter array_iter;
                 int index;
 
-                obj = JS_NewArrayObject(context, 0, JSVAL_NULL);
+                obj = JS_NewArrayObject(context, 0, NULL);
                 if (obj == NULL)
                     return JS_FALSE;
 
-                JS_AddRoot(context, &obj);
+                JS_AddObjectRoot(context, &obj);
                 dbus_message_iter_recurse(iter, &array_iter);
                 index = 0;
                 while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
                     jsval prop_value;
 
                     prop_value = JSVAL_VOID;
-                    JS_AddRoot(context, &prop_value);
+                    JS_AddValueRoot(context, &prop_value);
                     if (!gjs_js_one_value_from_dbus(context, &array_iter, &prop_value)) {
-                        JS_RemoveRoot(context, &prop_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &prop_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
                     if (!JS_DefineElement(context, obj,
                                           index, prop_value,
                                           NULL, NULL, JSPROP_ENUMERATE)) {
-                        JS_RemoveRoot(context, &prop_value);
-                        JS_RemoveRoot(context, &obj);
+                        JS_RemoveValueRoot(context, &prop_value);
+                        JS_RemoveObjectRoot(context, &obj);
                         return JS_FALSE;
                     }
 
-                    JS_RemoveRoot(context, &prop_value);
+                    JS_RemoveValueRoot(context, &prop_value);
                     dbus_message_iter_next(&array_iter);
                     index ++;
                 }
                 *value_p = OBJECT_TO_JSVAL(obj);
-                JS_RemoveRoot(context, &obj);
+                JS_RemoveObjectRoot(context, &obj);
             }
         }
         break;
@@ -257,7 +258,7 @@ gjs_js_one_value_from_dbus(JSContext       *context,
         {
             double v_DOUBLE;
             dbus_message_iter_get_basic(iter, &v_DOUBLE);
-            if (!JS_NewDoubleValue(context, v_DOUBLE, value_p))
+            if (!JS_NewNumberValue(context, v_DOUBLE, value_p))
                 return JS_FALSE;
         }
         break;
@@ -308,25 +309,27 @@ gjs_js_values_from_dbus(JSContext          *context,
     jsval value;
 
     value = JSVAL_VOID;
-    JS_AddRoot(context, &value);
+    JS_AddValueRoot(context, &value);
 
     *array_p = NULL;
 
     array = gjs_rooted_array_new();
 
-    do {
-        if (!gjs_js_one_value_from_dbus(context, iter, &value)) {
-            gjs_rooted_array_free(context, array, TRUE);
-            JS_RemoveRoot(context, &value);
-            return JS_FALSE; /* error message already set */
-        }
+    if (dbus_message_iter_get_arg_type(iter) != DBUS_TYPE_INVALID) {
+        do {
+            if (!gjs_js_one_value_from_dbus(context, iter, &value)) {
+                gjs_rooted_array_free(context, array, TRUE);
+                JS_RemoveValueRoot(context, &value);
+                return JS_FALSE; /* error message already set */
+            }
 
-        gjs_rooted_array_append(context, array, value);
-    } while (dbus_message_iter_next(iter));
+            gjs_rooted_array_append(context, array, value);
+        } while (dbus_message_iter_next(iter));
+    }
 
     *array_p = array;
 
-    JS_RemoveRoot(context, &value);
+    JS_RemoveValueRoot(context, &value);
 
     return JS_TRUE;
 }
@@ -773,11 +776,11 @@ append_dict(JSContext         *context,
         return JS_FALSE;
     }
 
-    prop_id = JSVAL_VOID;
+    prop_id = JSID_VOID;
     if (!JS_NextProperty(context, props_iter, &prop_id))
         return JS_FALSE;
 
-    while (prop_id != JSVAL_VOID) {
+    while (!JSID_IS_VOID(prop_id)) {
         jsval nameval;
         char *name;
         jsval propval;
@@ -861,7 +864,7 @@ append_dict(JSContext         *context,
         dbus_message_iter_close_container(&dict_iter, &entry_iter);
 
     next:
-        prop_id = JSVAL_VOID;
+        prop_id = JSID_VOID;
         if (!JS_NextProperty(context, props_iter, &prop_id))
             return JS_FALSE;
     }
